#========================================================================
# NOM: W3color
# ROL: Petit utilitaire en TCL pour afficher la dcomposition
#      en hexadcimal d'une couleur (pour pages Web, par exemple...)
#      Est fait pour fonctionner en Tk 8.0, mais a aussi t test en 4.1
#
#      Ce source est distribuable sans droit et sans autorisation !
#      S'il vous a servi, envoyez un mot  ankif@mygale.org
#
# AUT: AnkiF
# VER: 1.1
#
#========================================================================
#
# HST: 1.0  14/04/97  Cration
# HST: 1.1  02/07/97  Inversion ordre des sliders pour avoir R,V,B.
#                     remplacement "BACKGROUND" par "BGCOLOR"!
#
#========================================================================

#---------------------------------------------------
# Variables de l'application
#---------------------------------------------------

# Couleurs (composantes RVB)
set couleur_r(fond)  128
set couleur_v(fond)  128
set couleur_b(fond)  128

# Couleur de texte
set couleur_r(texte)    200
set couleur_v(texte)    0
set couleur_b(texte)    0

set couleur_r(lien)    0
set couleur_v(lien)    0
set couleur_b(lien)  200

# Couleur de lien visit
set couleur_r(lien_v)    0
set couleur_v(lien_v)  100
set couleur_b(lien_v)    0

# Autres variables globales
#   col_blue  = Variable associe au slider de slection de la composante Bleue
#               (change si le slider varie, et inversement)
#   col_green = Variable associe au slider de slection de la composante Verte
#               (change si le slider varie, et inversement)
#   col_red   = Variable associe au slider de slection de la composante Rouge
#               (change si le slider varie, et inversement)
#   exemple   = widget de texte contenant les exemples d'affichage
#   html      = widget de texte contenant l'instruction HTML
#   radioval  = variable associe aux boutons radio, vaut "fond", "texte", "lien", ou "lien_v"
#               correspond  l'lment courant en cours de modification grace aux sliders



#========================================================================
# FCT: CouleurCourante
# ROL: Retourne la couleur courante d'un des lments
# ARG: quoi       (E): "fond", "texte", "lien" ou "lien_v"
#
#========================================================================
proc CouleurCourante { quoi } {

upvar #0 couleur_r couleur_r
upvar #0 couleur_v couleur_v
upvar #0 couleur_b couleur_b
 
return [format #%02X%02X%02X $couleur_r($quoi) $couleur_v($quoi) $couleur_b($quoi) ]
}




#========================================================================
# FCT: InitScales
# ROL: Initialise la valeur des sliders en fonction de l'lment courant
#      (fond, texte, lien, lien visit), donn par la srie de radio boutons
#      Agit directement sur les variables globales
# ARG: nant
#
#========================================================================
proc InitScales { } {
 
  uplevel #0 {
   set col_red $couleur_r($radioval)
   set col_green $couleur_v($radioval)
   set col_blue $couleur_b($radioval)
   }
}




#========================================================================
# FCT: Colorie
# ROL: Colorie l'lment adquat avec les couleurs spcifie. L'lment
#      est donn par la valeur courante des boutons-radios (variable
#      'radioval')
# ARG: composante    (E): "Rouge", "Vert" ou "Bleu"
#      valeur        (E): valeur de la composante (0-255)
#
#========================================================================
proc Colorie { composante valeur } {
   global exemple
   global html
   upvar #0 couleur_r couleur_r
   upvar #0 couleur_v couleur_v
   upvar #0 couleur_b couleur_b
   upvar #0 radioval quoi
 
   # Mise  jour composante slectionne
   if { $composante == "Rouge" } { set couleur_r($quoi) $valeur }
   if { $composante == "Vert" } { set couleur_v($quoi) $valeur }
   if { $composante == "Bleu" } { set couleur_b($quoi) $valeur }

   # Calcul nouvelle couleur
   set color [CouleurCourante $quoi]

   # Mise  jour de l'objet courant
   if { $quoi == "fond" } { $exemple configure -background $color } 
   if { $quoi == "texte" } { $exemple tag configure coul_texte -foreground $color }
   if { $quoi == "lien" } { $exemple tag configure coul_lien -foreground $color }
   if { $quoi == "lien_v" } { $exemple tag configure coul_lien_v -foreground $color }

   # Mise  jour du texte en HTML
   $html configure -state normal
   $html delete 1.0 end
   $html insert end "<BODY TEXT=\"[CouleurCourante texte]\"\
                           LINK=\"[CouleurCourante lien]\"\
                           VLINK=\"[CouleurCourante lien_v]\"\
                           BGCOLOR=\"[CouleurCourante fond]\">"
   $html configure -state disabled
  }





#========================================================================
# FCT: Aide
# ROL: Affiche une petite fentre d'aide sans prtention
# ARG: nant
#
#========================================================================
proc Aide { } {
 
  catch { destroy .fenaide }
  toplevel .fenaide
  wm title .fenaide "Aide en ligne"

  set ptxtaide [frame .fenaide.panel1]
  set pcmdaide [frame .fenaide.panel2]

  scrollbar $ptxtaide.sb -orient vertical -command "$ptxtaide.text yview"
  text $ptxtaide.text -width 50 -wrap word -yscrollcommand "$ptxtaide.sb set"
  $ptxtaide.text tag configure coul_titres -background "#FFB0B0"

  $ptxtaide.text insert end "Prsentation\n" coul_titres
  $ptxtaide.text insert end "Ce programme permet de choisir les couleurs des lments principaux "
  $ptxtaide.text insert end "d'une page Web : couleur du fond, du texte, des liens\n\n"
  $ptxtaide.text insert end "Structure de l'cran\n" coul_titres
  $ptxtaide.text insert end "PARTIE SUPERIEURE\n"
  $ptxtaide.text insert end "- Elment  modifier: slectionne l'lment en cours de modification, c'est  dire "
  $ptxtaide.text insert end "celui qui change de couleur par action sur les jauges\n"
  $ptxtaide.text insert end "- Slection de la couleur: les trois jauges permettent de rgler "
  $ptxtaide.text insert end "la couleur de l'lment courant. L'effet est visible dans la zone d'exemple de "
  $ptxtaide.text insert end "la partie centrale\n\n"
  $ptxtaide.text insert end "PARTIE CENTRALE\nElle contient :\n"
  $ptxtaide.text insert end "- la zone d'exemple montrant l'effet des modifications sur les diffrents lments\n"
  $ptxtaide.text insert end "- la zone contenant l'instruction HTML  fournir  un butineur Web pour obtenir "
  $ptxtaide.text insert end "le mme effet visuel\n\n"
  $ptxtaide.text insert end "Choix de l'lment pour rglage de la couleur\n" coul_titres
  $ptxtaide.text insert end "Slectionner le bouton radio correspondant, ou cliquer dans la zone "
  $ptxtaide.text insert end "d'exemples sur le texte correspondant\n\n"
  $ptxtaide.text insert end "Rcupration des modifications\n" coul_titres
  $ptxtaide.text insert end "Appuyer sur le bouton 'Copie' :"
  $ptxtaide.text insert end "le texte est recopi dans le presse-papiers, et peut tre "
  $ptxtaide.text insert end "coll dans votre diteur HTML prfr\n"
  pack $ptxtaide.sb -side right -expand yes -fill y
  pack $ptxtaide.text -side left -expand yes -fill both

  button $pcmdaide.butt -text "OK" -command { destroy .fenaide}
  pack $pcmdaide.butt

  pack $ptxtaide $pcmdaide  -side top
 }









#========================================================================
# FCT: ---
# ROL: Programme principal - Cre les widgets et les fentres...
# ARG: nant
#
#========================================================================


#---------------------------------------------------
# Dfinition des 4 parties de la fentre :
# - partie suprieure = radio-boutons et sliders
# - partie mdiane 1  = exemples des textes
# - partie mdiane 2  = HTML gnr
# - partie infrieure = boutons
#---------------------------------------------------
set pcomm [frame .panel1]
set ptext [frame .panel2]
set phtml [frame .panel3]
set pbout [frame .panel4]


#---------------------------------------------------
# Groupe des radio-boutons : pour avoir un joli 
# aspect quelles que soient les circonstances, on
# regroupe les radio-boutons dans un petit frame,
# et on cre un frame parent dans lequel on case le
# titre et ce petit frame.
#---------------------------------------------------
# -- le frame englobant, avec une zoulie bordure
set radio_panel [frame $pcomm.radios -border 2 -relief groove]

# -- le titre. On le cre puis on le passe en gras
label $radio_panel.titre -text "Elment  modifier"
if { $tk_version >= 8.0 } \
  {
   set police [$radio_panel.titre cget -font]
   font create police_titres -family [lindex $police 0] -size [lindex $police 1] -weight bold
   $radio_panel.titre config -font police_titres
  }

# -- le petit frame des radio-boutons
frame $radio_panel.tiframe

# -- les radio-boutons dans ce petit frame
set radioval lien
set rb1 [radiobutton $radio_panel.tiframe.rb1 -value fond -variable radioval \
         -text "Fond d'cran" -command InitScales ]
set rb2 [radiobutton $radio_panel.tiframe.rb2 -value texte -variable radioval \
         -text "Texte" -command InitScales ]
set rb3 [radiobutton $radio_panel.tiframe.rb3 -value lien -variable radioval \
         -text "Lien" -command InitScales ]
set rb4 [radiobutton $radio_panel.tiframe.rb4 -value lien_v -variable radioval \
         -text "Lien visit" -command InitScales ]
pack $rb1 $rb2 $rb3 $rb4 -side top -anchor w

pack $radio_panel.titre -side top -anchor w
pack $radio_panel.tiframe -side bottom -fill both -expand yes -anchor w -padx 5 -pady 10


#---------------------------------------------------
# Le groupe des scales, idem
#---------------------------------------------------
# Un petit cadre sans relief autour du frame
set scale_panel [frame $pcomm.scales -border 2 -relief groove]

# Titre
label $scale_panel.titre -text "Slection de la couleur"
if { $tk_version >= 8.0 } \
  {
   $scale_panel.titre config -font police_titres
  }

# Chaque slider est cr dans un petit frame car si on utilise
# le label intrinsque, il se trouve au dessus : pas beau
set slider1 [frame $scale_panel.slider1]
label $slider1.titre -text "Rouge"
set sc1 [scale $slider1.sc -orient horizontal -from 0 -to 255 \
         -variable col_red \
         -command "Colorie Rouge"]

set slider2 [frame $scale_panel.slider2]
label $slider2.titre -text "Vert"
set sc2 [scale $slider2.sc -orient horizontal -from 0 -to 255 \
         -variable col_green \
         -command "Colorie Vert"]

set slider3 [frame $scale_panel.slider3]
label $slider3.titre -text "Bleu"
set sc3 [scale $slider3.sc -orient horizontal -from 0 -to 255 \
         -variable col_blue \
         -command "Colorie Bleu"]


pack $slider1.titre $sc1 -side left -anchor s
pack $slider2.titre $sc2 -side left -anchor s
pack $slider3.titre $sc3 -side left -anchor s
pack $scale_panel.titre -side top -anchor w
pack $slider3 $slider2 $slider1 -side bottom -anchor e -padx 10



#---------------------------------------------------
# Arrangement des 2 groupes (boutons radio et scales)
# dans la partie suprieure
#---------------------------------------------------
pack $radio_panel $scale_panel -side left -padx 10 -expand yes -fill both

pack $pcomm


#---------------------------------------------------
# La fentre d'exemple de textes
#---------------------------------------------------
set exemple [text $ptext.text -height 3 -width 45 -wrap char]

$exemple configure -bg [CouleurCourante fond]

# Cration des tags permettant de modifier les couleurs de texte
# de manire slective

$exemple tag configure coul_fond -foreground [CouleurCourante fond]
$exemple tag configure coul_texte -foreground [CouleurCourante texte]
$exemple tag configure coul_lien -foreground [CouleurCourante lien]
$exemple tag configure coul_lien_v -foreground [CouleurCourante lien_v]


# Si on slectionne les textes, c'est comme si on slectionnait
# le radio-boutons correspondant. Idem pour le fond.

$exemple tag bind coul_fond <ButtonPress-1> { $rb1 invoke }
$exemple tag bind coul_texte <ButtonPress-1> { $rb2 invoke }
$exemple tag bind coul_lien <ButtonPress-1> { $rb3 invoke }
$exemple tag bind coul_lien_v <ButtonPress-1> { $rb4 invoke }


# Insertion du texte puis on le rend "read only"
# On positionne chaque tag sur le texte auquel il correspond.
# Le tag "couleur de fond" est associ aux <CR> pour correspondre
# aux parties sans texte (astuce)

$exemple insert end "Ceci est la couleur du texte" coul_texte
$exemple insert end "\n" coul_fond
$exemple insert end "Ceci est la couleur d'un lien" coul_lien
$exemple insert end "\n" coul_fond
$exemple insert end "Ceci est la couleur d'un lien visit" coul_lien_v
$exemple insert end "\n" coul_fond
$exemple configure -state disabled


#---------------------------------------------------
# Arrangement zone de texte dans la partie mdiane
#---------------------------------------------------
pack $exemple -expand yes -fill x


#---------------------------------------------------
# La fentre de texte HTML
#---------------------------------------------------
set html [text $phtml.text -width 50 -height 2]

$html configure -state disabled

button $phtml.clipbrd -text "Copie" \
                      -command { clipboard clear ; clipboard append [$phtml.text get 1.0 end]}


#---------------------------------------------------
# Arrangement zone de texte dans son panel
#---------------------------------------------------
pack $phtml.clipbrd -side right
pack $html -side left -expand yes -fill x


#---------------------------------------------------
# Les boutons
#---------------------------------------------------
button $pbout.butt1 -text "Fermer" -command "destroy ."
button $pbout.butt2 -text "Aide..." -command "Aide"


#---------------------------------------------------
# Arrangement boutons dans la partie infrieure
#---------------------------------------------------
pack $pbout.butt1 $pbout.butt2 -pady 5 -padx 10 -side left


#---------------------------------------------------
# Arrangement des 4 parties de la fentre
#---------------------------------------------------
# d'abord les trois du haut...
pack $pcomm $ptext $phtml -side top -expand yes -fill x -pady 5

# ... puis les boutons sparment pour tre centrs
pack $pbout -side top -expand no

. configure -width 200
wm title . "W3color: choix des couleurs d'lments d'une page Web"


#---------------------------------------------------
# Slection initiale = couleur de lien
#---------------------------------------------------
$rb2 invoke
# 



